@@ -140,37 +140,74 @@ class SEOContext( BrowserView ):
         html_comment = self.getSEOProperty( 'qSEO_html_comment' )
         return html_comment and html_comment or '' 
 
-    def seo_keywords( self ):
-        """ Generate Keywords from SEO properties.
+    def meta_keywords( self ):
+        """ Generate Meta Keywords from SEO properties (global and local) with Subject,
+            depending on the options in configlet.
         """
         prop_name = 'qSEO_keywords'
-        add_keywords = 'additional_keywords'
         accessor = 'Subject'
         context = aq_inner(self.context)
-
         keywords = Set([])
-        if context.hasProperty(prop_name):
-            keywords = Set(context.getProperty(prop_name))
-
         pprops = getToolByName(context, 'portal_properties')
         sheet = getattr(pprops, 'seo_properties', None)
-        if sheet and sheet.hasProperty(add_keywords):
-            keywords = keywords | Set(sheet.getProperty(add_keywords))
-
-        if keywords:
-            return keywords
-
         method = getattr(context, accessor, None)
         if not callable(method):
             return None
 
         # Catch AttributeErrors raised by some AT applications
         try:
-            value = method()
+            subject = Set(method())
         except AttributeError:
-            value = None
+            subject = keywords
 
-        return value
+        if sheet:
+          settings_use_keywords_sg = sheet.getProperty('settings_use_keywords_sg')
+          settings_use_keywords_lg = sheet.getProperty('settings_use_keywords_lg')
+          global_keywords = Set(sheet.getProperty('additional_keywords', None))
+          local_keywords = Set(context.getProperty(prop_name, None))
+          # Subject overrides global seo keywords and global overrides local seo keywords
+          if [settings_use_keywords_sg, settings_use_keywords_lg] == [1, 1]:
+              keywords = subject
+          # Subject overrides global seo keywords and merge global and local seo keywords
+          elif [settings_use_keywords_sg, settings_use_keywords_lg] == [1, 2]:
+              keywords = subject | local_keywords
+          # Global seo keywords overrides Subject and global overrides local seo keywords
+          elif [settings_use_keywords_sg, settings_use_keywords_lg] == [2, 1]:
+              keywords = global_keywords
+          # Global seo keywords overrides Subject and merge global and local seo keywords
+          elif [settings_use_keywords_sg, settings_use_keywords_lg] == [2, 2]:
+              keywords = global_keywords | local_keywords
+          # Merge Subject and global seo keywords and global overrides local seo keywords
+          elif [settings_use_keywords_sg, settings_use_keywords_lg] == [3, 1]:
+              keywords = subject | global_keywords
+          # Merge Subject and global seo keywords and merge global and local seo keywords
+          elif [settings_use_keywords_sg, settings_use_keywords_lg] == [3, 2]:
+              keywords = subject | global_keywords | local_keywords
+          else:
+              keywords = subject
+        else:
+            keywords = subject
+
+        return tuple(keywords)
+
+    def seo_keywords( self ):
+        """ Generate SEO Keywords from SEO properties (global merde local).
+        """
+        prop_name = 'qSEO_keywords'
+        context = aq_inner(self.context)
+        keywords = Set([])
+        pprops = getToolByName(context, 'portal_properties')
+        sheet = getattr(pprops, 'seo_properties', None)
+
+        if sheet:
+            settings_use_keywords_sg = sheet.getProperty('settings_use_keywords_sg')
+            settings_use_keywords_lg = sheet.getProperty('settings_use_keywords_lg')
+            global_keywords = Set(sheet.getProperty('additional_keywords', None))
+            local_keywords = Set(context.getProperty(prop_name, None))
+            keywords = global_keywords | local_keywords
+        else:
+            keywords = ''
+        return tuple(keywords)
 
     def seo_canonical( self ):
         """ Generate canonical URL from SEO properties.
@@ -211,13 +248,6 @@ class SEOControlPanel( ControlPanelView ):
         """
         return filter(lambda x:x.id == 'seo_properties', type_info.listActions())
 
-
-    def isEnableCT(self, type_id):
-        """
-        """
-        seo_props = getToolByName(self.portal_properties, 'seo_properties')
-        return type_id in seo_props.getProperty('content_types_seoprops_enabled', '')
-
     def test( self, condition, first, second ):
         """
         """
@@ -247,6 +277,12 @@ class SEOControlPanel( ControlPanelView ):
         seo = self.portal_properties.seo_properties
         return seo.getProperty('additional_keywords')
 
+    def getFilterKeywordsByContent( self ):
+        """ Get values from filter_keywords_by_content property in seo_properties.
+        """
+        seo = self.portal_properties.seo_properties
+        return seo.getProperty('filter_keywords_by_content')
+
     def createMultiColumnList( self ):
         """
         """
@@ -268,6 +304,9 @@ class SEOControlPanel( ControlPanelView ):
         additionalKeywords = request.get('additionalKeywords', [])
         default_custom_metatags = request.get('default_custom_metatags', [])
         metatags_order = request.get('metatags_order', [])
+        filterKeywordsByContent = request.get('filterKeywordsByContent', None)
+        settingsUseKeywordsSG = int(request.get('settingsUseKeywordsSG', 1))
+        settingsUseKeywordsLG = int(request.get('settingsUseKeywordsLG', 1))
 
         site_props = getToolByName(self.portal_properties, 'site_properties')
         seo_props = getToolByName(self.portal_properties, 'seo_properties')
@@ -281,6 +320,9 @@ class SEOControlPanel( ControlPanelView ):
             seo_props.manage_changeProperties(default_custom_metatags=default_custom_metatags)
             seo_props.manage_changeProperties(metatags_order=metatags_order)
             seo_props.manage_changeProperties(content_types_seoprops_enabled=content_types_seoprops_enabled)
+            seo_props.manage_changeProperties(filter_keywords_by_content=filterKeywordsByContent)
+            seo_props.manage_changeProperties(settings_use_keywords_sg=settingsUseKeywordsSG)
+            seo_props.manage_changeProperties(settings_use_keywords_lg=settingsUseKeywordsLG)
 
             for ptype in self.portal_types.objectValues():
                 acts = filter(lambda x: x.id == 'seo_properties', ptype.listActions())
@@ -308,6 +350,20 @@ class SEOControlPanel( ControlPanelView ):
         """
         return self.portal_types.getTypeInfo( type_name )
 
+    def select_settings_use_keywords_sg(self):
+        context = aq_inner(self.context)
+        site_properties = getToolByName(context, 'portal_properties')
+        if hasattr(site_properties, 'seo_properties'):
+            settings_use_keywords_sg = getattr(site_properties.seo_properties, 'settings_use_keywords_sg', 0)
+        return settings_use_keywords_sg
+
+    def select_settings_use_keywords_lg(self):
+        context = aq_inner(self.context)
+        site_properties = getToolByName(context, 'portal_properties')
+        if hasattr(site_properties, 'seo_properties'):
+            settings_use_keywords_lg = getattr(site_properties.seo_properties, 'settings_use_keywords_lg', 0)
+        return settings_use_keywords_lg
+
 
 class SEOContextPropertiesView( BrowserView ):
     """ This class contains methods that allows to manage seo properties.
